# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/common.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/common.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x13protos/common.proto\"\x1c\n\nBoolResult\x12\x0e\n\x06result\x18\x01 \x01(\x08\",\n\x08MarketId\x12\x10\n\x08\x65xchange\x18\x01 \x01(\t\x12\x0e\n\x06symbol\x18\x02 \x01(\t\"#\n\x0bSessionHash\x12\x14\n\x0csession_hash\x18\x01 \x01(\tb\x06proto3')
)




_BOOLRESULT = _descriptor.Descriptor(
  name='BoolResult',
  full_name='BoolResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='BoolResult.result', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=23,
  serialized_end=51,
)


_MARKETID = _descriptor.Descriptor(
  name='MarketId',
  full_name='MarketId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='exchange', full_name='MarketId.exchange', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='symbol', full_name='MarketId.symbol', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=53,
  serialized_end=97,
)


_SESSIONHASH = _descriptor.Descriptor(
  name='SessionHash',
  full_name='SessionHash',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='session_hash', full_name='SessionHash.session_hash', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=99,
  serialized_end=134,
)

DESCRIPTOR.message_types_by_name['BoolResult'] = _BOOLRESULT
DESCRIPTOR.message_types_by_name['MarketId'] = _MARKETID
DESCRIPTOR.message_types_by_name['SessionHash'] = _SESSIONHASH
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BoolResult = _reflection.GeneratedProtocolMessageType('BoolResult', (_message.Message,), {
  'DESCRIPTOR' : _BOOLRESULT,
  '__module__' : 'protos.common_pb2'
  # @@protoc_insertion_point(class_scope:BoolResult)
  })
_sym_db.RegisterMessage(BoolResult)

MarketId = _reflection.GeneratedProtocolMessageType('MarketId', (_message.Message,), {
  'DESCRIPTOR' : _MARKETID,
  '__module__' : 'protos.common_pb2'
  # @@protoc_insertion_point(class_scope:MarketId)
  })
_sym_db.RegisterMessage(MarketId)

SessionHash = _reflection.GeneratedProtocolMessageType('SessionHash', (_message.Message,), {
  'DESCRIPTOR' : _SESSIONHASH,
  '__module__' : 'protos.common_pb2'
  # @@protoc_insertion_point(class_scope:SessionHash)
  })
_sym_db.RegisterMessage(SessionHash)


# @@protoc_insertion_point(module_scope)
